/*
    Copyright (c) 2010 Andreas Holzammer <andreas.holzammer@kdab.com>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include <QtPlugin>

Q_IMPORT_PLUGIN(akonadi_ical_resource)
Q_IMPORT_PLUGIN(akonadi_vcard_resource)
Q_IMPORT_PLUGIN(akonadi_maildir_resource)
Q_IMPORT_PLUGIN(akonadi_akonotes_resource)
Q_IMPORT_PLUGIN(akonadi_contacts_resource)
Q_IMPORT_PLUGIN(akonadi_strigi_feeder)
Q_IMPORT_PLUGIN(korgac)
Q_IMPORT_PLUGIN(newmailnotifieragent)

Q_IMPORT_PLUGIN(akonadi_serializer_mail)
Q_IMPORT_PLUGIN(akonadi_serializer_addressee)
Q_IMPORT_PLUGIN(akonadi_serializer_contactgroup)
Q_IMPORT_PLUGIN(akonadi_serializer_kcalcore)

extern int cemain( int argc, char ** argv );

int main( int argc, char ** argv )
{
  return cemain( argc, argv );
}
